#ifndef __CONNECT_H__

   #include <psapi.h>

   class CPsConnection {
   public:
   // Construction/Destruction
      CPsConnection() ;
      ~CPsConnection() ;
      
   // Connection
      BOOL Open(LPCSTR lpszModelName, HWND hwndApp, BOOL bBrowse) ;
      void Close() ;
      BOOL IsConnected() { return (m_hConnection != NULL) ; }
      LPCSTR GetModelName() const { return m_szModelPath ; }
		BOOL EnableWantPauseNotifications(BOOL bEnable);   // BB960301: Added
      
   // Parameter list handling
      int AppendPar(LPCSTR lpszParName) ;
      BOOL RemoveParAt(int iPar) const ;
      BOOL RemoveAllPar() const ;
      int SearchPar(LPCSTR lpszParName) const ;
      int GetParCount() const ;

   // Parameter observers
      LPSTR GetParName(int iPar, LPSTR lpszName, UINT uiSizeOfName) const ;
      BOOL GetParValue(int iPar, double *pdbValue) const ;
      LPSTR GetParUnit(int iPar, LPSTR lpszUnit, UINT uiSizeOfUnit) const ;
      BOOL GetParScale(int iPar, double FAR *pdbMin, double FAR *pdbMax) const ;

   // Parameter operators
      BOOL SetParValue(int iPar, double dbValue) ;

   // Simulation control
      BOOL Play(BOOL bPlayOn) ;
      BOOL PlayStep(long cSteps) ;
      BOOL Pause(BOOL bPauseOn) ;
      void SetStartTime(double dbStartTime) ;
      void SetStopTime(double dbStopTime) ;
      void SetTimeStep(double dbTimeStep) ;
      void SetTimeDelay(double dbTimeDelay) ;
      void SetPauseAtTime(double dbPauseTime) ;
	
	// Simulation observers
      BOOL IsRunning() const ;
      BOOL IsPaused() const ;
      double GetStartTime() const { return m_dbStartTime ; }
      double GetStopTime() const { return m_dbStopTime ; }
      double GetTimeStep() const { return m_dbTimeStep ; }
      double GetTimeDelay() const { return m_dbTimeDelay; }
      double GetPauseAtTime() const { return m_dbPauseTime; }
      double GetTime() const { return m_dbTime ; }

   // Vectorization methods
      BOOL BeginChangeRanges() ;
      BOOL EndChangeRanges() ;
      BOOL SetRange(LPCSTR lpszRangeName, short nFirst, short nLast) ;

	// Model observers
		int GetModelCount() const ;
		LPSTR GetModelName(int iModel, LPSTR lpszName, UINT cbNameBuf) const ;

		int GetVariableCount(int iModel) const ;
		LPSTR GetVariableName(int iModel, int iVariable, 
			LPSTR lpszName, UINT cbNameBuf) ;

		int VariableHasInputs(int iModel, int iVariable) const ;
		int GetVariableType(int iModel, int iVariable) const ;

		LPSTR GetElementName(int iModel, int iVariable, 
			int iElement, LPSTR lpszName, UINT cbNameBuf) const ;
		int GetElementCount(int iModel, int iVariable) const ;

	// Notification handler
      static LRESULT Notify(LPARAM lParam) ;

   // Notification overridables
      virtual BOOL NotifyPreBeginSim() ;
      virtual BOOL NotifyBeginSim() ;
      virtual void NotifyEndSim() ;
      virtual void NotifyOpenModel() ;
      virtual void NotifyCloseModel() ;
      virtual void NotifyUpdateValues(BOOL bFinal) ;
      virtual void NotifyParamDeleted(int iPar) ;
      virtual void NotifyPauseOn();   // BB960301: Added
      virtual void NotifyPauseOff();  // BB960301: Added
      virtual BOOL NotifyWantPause(); // BB960301: Added

   private:
      enum SimState { SimStopped, SimRunning } ;

      char        m_szModelPath[256] ; // Path of the model
      HINSTANCE   m_hinst ;            // instance of application
      HWND        m_hwndApp ;          // Window to receive notifications
      HCONNECTION m_hConnection ;      // Handle of the connection
      double      m_dbStartTime ;      // StartTime of model
      double      m_dbStopTime ;       // Stop time of model
      double      m_dbTimeStep ;       // Time step of model
      double      m_dbTimeDelay ;      // Delay time of model
      double      m_dbTime ;           // Current time
      double FAR* m_ValueTab ;         // Table of values only accessible on UpdateValues
      int         m_cValuesInTab ;     // The number of values in previous Tab

      SimState    m_State ;            // Current simulation state
      BOOL        m_bPaused ;          // Is the simulation paused
		double		m_dbPauseTime;

      BOOL VerifyOpen(BOOL bBrowse = FALSE) ;
      void UpdateCtrlParams() ;
      void SetTimeSpec(BOOL bUpdatePause);
   } ; // CPsConnection



#define __CONNECT_H__
#endif
