unit Demo;
{
  File:              Apidemo.pas
  Full name:         Application Programming Interface Demo
  Author:            Trond Leikvoll
  Copyright:         (c) 1996 Powersim AS
  Purpose:           Show how the new Powersim component for Delphi can be used
                     to make a user interface for a powersim model.
  Changes:           When     Who      What
                     960301   TL       Created
}
interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Buttons, StdCtrls, VBXCtrl, Pict, Spin, TabNotBk,
  ExtCtrls, Chart2fx, PsCnct, Gauges;

type
  TDemoForm = class(TForm)
   {Visual objects, created designtime}
    MainTabbedNotebook: TTabbedNotebook;
    SetupGroupBox: TGroupBox;
    StartTimeLabel: TLabel;
    StopTimeLabel: TLabel;
    TimeDelayLabel: TLabel;
    TimeStepLabel: TLabel;
    StartEdit: TEdit;
    DelayEdit: TEdit;
    StepEdit: TEdit;
    SpinButtonStart: TSpinButton;
    SpinButtonStop: TSpinButton;
    SpinButtonDelay: TSpinButton;
    SpinButtonStep: TSpinButton;
    StopEdit: TEdit;
    ParametersGroupBox: TGroupBox;
    ParametersListBox: TListBox;
    NumberOfParameters: TLabel;
    ParCountEdit: TEdit;
    Salespeople: TLabel;
    SalesPeopleLabel: TLabel;
    OrdersLabel: TLabel;
    OrdersBooked: TLabel;
    BudgetLabel: TLabel;
    Budget: TLabel;
    BacklogLabel: TLabel;
    BacklogBiPict: TBiPict;
    Backlog: TLabel;
    DelayLabel: TLabel;
    DeliveryDelayBiPict: TBiPict;
    DeliveryDelayRecognised: TLabel;
    SalesEffLabel: TLabel;
    SalesEffectiveness: TLabel;
    RevenueToSalesGroupBox: TGroupBox;
    Revenue_To_SalesLabel: TLabel;
    Revenue_To_SalesScrollBar: TScrollBar;
    SalespeopleGroupBox: TGroupBox;
    SalaryLabel: TLabel;
    SalaryScrollBar: TScrollBar;
    AdjustmentLongRadioButton: TRadioButton;
    AdjustmentMediumRadioButton: TRadioButton;
    AdjustmentShortRadioButton: TRadioButton;
    AdjustmentTimeLabel2: TLabel;
    Salary: TLabel;
    SimulationControlsGroupBox: TGroupBox;
    OrdersBookedBiPict: TBiPict;
    BudgetBiPict: TBiPict;
    PlaySpeedButton: TSpeedButton;
    PauseSpeedButton: TSpeedButton;
    StopSpeedButton: TSpeedButton;
    StepSpeedButton: TSpeedButton;
    PsMarket: TPsConnection;
    SalespeopleChartFX: TChartFX;
    ModelTitleLabel: TLabel;
    TimeGauge: TGauge;
    SimulatedTime: TLabel;
    TimeLabel: TLabel;
    AdjustmentTimeLabel: TLabel;
    LargeSalesPeopleImage: TImage;
    SalesEffectivenessGauge: TGauge;
    CauseLoop3Image: TImage;
    CauseLoop4Image: TImage;
    SmallSalespeopleImage: TImage;
    AutoPauseEdit: TEdit;
    AutoPauseSpinButton: TSpinButton;
    every: TLabel;
    Step: TLabel;
    AutoPauseCheckBox: TCheckBox;
    BacklogChartFX: TChartFX;
    CauseLoop1Image: TImage;
    CauseLoop2Image: TImage;
    CloseBitBtn: TBitBtn;
    Memo1: TMemo;

  { Methods for handling user commands }

    procedure SpinButtonStartDownClick(Sender: TObject);
    procedure SpinButtonStartUpClick(Sender: TObject);
    procedure SpinButtonStopUpClick(Sender: TObject);
    procedure SpinButtonStopDownClick(Sender: TObject);
    procedure SpinButtonDelayUpClick(Sender: TObject);
    procedure SpinButtonDelayDownClick(Sender: TObject);
    procedure SpinButtonStepDownClick(Sender: TObject);
    procedure SpinButtonStepUpClick(Sender: TObject);
    procedure MainTabbedNotebookEnter(Sender: TObject);
    procedure PlaySpeedButtonClick(Sender: TObject);
    procedure PauseSpeedButtonClick(Sender: TObject);
    procedure StopSpeedButtonClick(Sender: TObject);
    procedure StepSpeedButtonClick(Sender: TObject);
    procedure SalaryScrollBarScroll(Sender: TObject;
      ScrollCode: TScrollCode; var ScrollPos: Integer);
    procedure Revenue_To_SalesScrollBarScroll(Sender: TObject;
      ScrollCode: TScrollCode; var ScrollPos: Integer);
    procedure AdjustmentLongRadioButtonEnter(Sender: TObject);
    procedure AdjustmentMediumRadioButtonEnter(Sender: TObject);
    procedure AdjustmentShortRadioButtonEnter(Sender: TObject);
    procedure AutoPauseCheckBoxEnter(Sender: TObject);
    procedure AutoPauseSpinButtonDownClick(Sender: TObject);
    procedure AutoPauseSpinButtonUpClick(Sender: TObject);
    procedure LargeSalesPeopleImageClick(Sender: TObject);
    procedure BacklogBiPictClick(Sender: TObject);
    procedure StepEditExit(Sender: TObject);
    procedure CloseBitBtnClick(Sender: TObject);
    procedure SalaryScrollBarChange(Sender: TObject);
    procedure Revenue_To_SalesScrollBarChange(Sender: TObject);

  { System dependent events, such as PreBeginSim, Activate, Create etc.}

    procedure FormActivate(Sender: TObject);
    procedure PsMarketUpdateValues(Sender: TObject; Final: Boolean);
    function PsMarketPreBeginSim(Sender: TObject): Boolean;
    procedure PsMarketEndSim(Sender: TObject);
    function PsMarketBeginSim(Sender: TObject): Boolean;
    procedure PsMarketCloseModel(Sender: TObject);
    procedure PsMarketOpenModel(Sender: TObject);
  end;

var
  DemoForm: TDemoForm;

implementation

var
 {Declaration of local control parameters for the simulation}
  AdjustmentTime, BacklogMaxValue, SalespeopleMaxValue : Double;
  AutoPauseStep,StepsExecuted : Integer;
  AutoPauseOn,UpdateOnEveryStep : Boolean;

 {Enumerating the values array of the Powersim connection}

  Parameter_Revenue_To_Sales,Parameter_Salespeople_Salary,Parameter_Backlog,
  Parameter_Salespeople,Parameter_Orders_Booked,Parameter_Budget,
  Parameter_Sales_Effectiveness,Parameter_Salespeople_Adjustment_Time,
  Parameter_Delivery_Delay_Recognized : Integer;


{$R *.DFM}

procedure TDemoForm.MainTabbedNotebookEnter(Sender: TObject);
{  Purpose:       Read setup parameters from Powersim.
   In:            Sender - Standard eventhandler parameter for Delphi
   Out:
   Return:
}
begin
  {Initialize SetupPage Editboxes}
   StartEdit.Text := FloatToStr(PsMarket.StartTime);
   StopEdit.Text := FloatToStr(PsMarket.StopTime);
   DelayEdit.Text := FloatToStr(PsMarket.TimeDelay);
   StepEdit.Text := FloatToStr(PsMarket.TimeStep);
end;


procedure TDemoForm.SpinButtonStartDownClick(Sender: TObject);
{  Purpose:       Decrease Powersim model's StartTime by one.  Display the
                  new current StartTime
   In:            Sender - Standard eventhandler parameter for Delphi
   Out:
   Return:
}
var
  CurrentValue : Double;
begin
   CurrentValue := StrToFloat(StartEdit.Text);
   CurrentValue := CurrentValue - 1;
   StartEdit.Text := FloatToStr(CurrentValue);
   PsMarket.StartTime := CurrentValue;
end;


procedure TDemoForm.SpinButtonStartUpClick(Sender: TObject);
{  Purpose:       Increase Powersim model's StartTime by one.  Display the
                  new current StartTime.  (PS StartTime should be icremented
                  only if its new value will be smaller than StopTime
   In:            Sender - Standard eventhandler parameter for Delphi
   Out:
   Return:
}
var
  CurrentValue : Double;
begin
   CurrentValue := StrToFloat(StartEdit.Text);
   if (CurrentValue+1) < StrToFloat(StopEdit.Text) then begin
     CurrentValue := CurrentValue + 1;
     StartEdit.Text := FloatToStr(CurrentValue);
     PsMarket.StartTime := CurrentValue;
   end;
end;

procedure TDemoForm.SpinButtonStopUpClick(Sender: TObject);
{  Purpose:       Increase Powersim model's StopTime by one.  Display the
                  new current StopTime.
   In:            Sender - Standard eventhandler parameter for Delphi
   Out:
   Return:
}
var
  CurrentValue : Double;
begin
   CurrentValue := StrToFloat(StopEdit.Text);
   CurrentValue := CurrentValue + 1;
   StopEdit.Text := FloatToStr(CurrentValue);
   PsMarket.StopTime := CurrentValue;
end;

procedure TDemoForm.SpinButtonStopDownClick(Sender: TObject);
{  Purpose:       Decrease Powersim model's StopTime by one.  Display the
                  new current StopTime.  (PS StopTime should be decremented
                  only if its new value will be greater than StartTime
   In:            Sender - Standard eventhandler parameter for Delphi
   Out:
   Return:
}
var
  CurrentValue : Double;
begin
   CurrentValue := StrToFloat(StopEdit.Text);
   if (CurrentValue-1) > StrToFloat(StartEdit.Text) then begin
     CurrentValue := CurrentValue - 1;
     StopEdit.Text := FloatToStr(CurrentValue);
     PsMarket.StopTime := CurrentValue;
   end;
end;

procedure TDemoForm.SpinButtonDelayUpClick(Sender: TObject);
{  Purpose:       Increase Powersim model's TimeDelay by one.  Display the
                  new current TimeDelay.
   In:            Sender - Standard eventhandler parameter for Delphi
   Out:
   Return:
}
var
  CurrentValue : Double;
begin
   CurrentValue := StrToFloat(DelayEdit.Text) + 1;
   DelayEdit.Text := FloatToStr(CurrentValue);
   PsMarket.TimeDelay := CurrentValue;
end;

procedure TDemoForm.SpinButtonDelayDownClick(Sender: TObject);
{  Purpose:       Decrease Powersim model's TimeDelay by one.  Display the
                  new current TimeDelay. (PS TimeDelay should be greater than
                  or equal to zero)
   In:            Sender - Standard eventhandler parameter for Delphi
   Out:
   Return:
}
var
  CurrentValue : Double;
begin
   CurrentValue := StrToFloat(DelayEdit.Text);
   if (CurrentValue-1) >= 0 then begin
     CurrentValue := CurrentValue - 1;
     DelayEdit.Text := FloatToStr(CurrentValue);
     PsMarket.TimeDelay := CurrentValue;
   end;
end;

procedure TDemoForm.SpinButtonStepDownClick(Sender: TObject);
{  Purpose:       Decrease Powersim model's TimeStep to half its original value.
                  Display the new current TimeStep.
   In:            Sender - Standard eventhandler parameter for Delphi
   Out:
   Return:
}
var
  CurrentValue : Double;
begin
   CurrentValue := StrToFloat(StepEdit.Text);
   CurrentValue := CurrentValue/2;
  {Check if the graph should be updated on every simulated step.  This property
  is turned off when TimeStep is less than one}
   if CurrentValue < 1 then
     UpdateOnEveryStep := False;
   StepEdit.Text := FloatToStr(CurrentValue);
   PsMarket.TimeStep := CurrentValue;
end;

procedure TDemoForm.SpinButtonStepUpClick(Sender: TObject);
{  Purpose:       Increase Powersim model's TimeStep to twice its original
                  value.  Display the new current TimeStep.
   In:            Sender - Standard eventhandler parameter for Delphi
   Out:
   Return:
}
var
  CurrentValue : Double;
begin
   CurrentValue := StrToFloat(StepEdit.Text);
   CurrentValue := CurrentValue*2;
   if CurrentValue >= 1 then
     UpdateOnEveryStep := True;
   StepEdit.Text := FloatToStr(CurrentValue);
   PsMarket.TimeStep := CurrentValue;
end;

procedure TDemoForm.FormActivate(Sender: TObject);
{  Purpose:       Eventhandler for the TDemoForm's FormActivate
   In:            Sender - Standard eventhandler parameter for Delphi
   Out:
   Return:
}
var
  i : integer;
begin
  {Initialize Labels and Gauges with Powersim default values}
   TimeGauge.MinValue := Round(PsMarket.StartTime);
   TimeGauge.MaxValue := Round(PsMarket.StopTime);
   TimeGauge.Progress := Round(PsMarket.StartTime);
   TimeLabel.Caption := IntToStr(TimeGauge.Progress) + ' Weeks';
  {Initialize Unit's control parameters}
   AutoPauseStep := 1;
   AdjustmentTime := 20;
   AutopauseOn := False;
   UpdateOnEveryStep := True;
  {Initialize Notebooks to display 'Control' page }
   MainTabbedNotebook.ActivePage := 'Control';
  {Show model name }
   ModelTitleLabel.Caption := 'Powersim Model : ' + PsMarket.ModelName;
end;

procedure TDemoForm.PlaySpeedButtonClick(Sender: TObject);
{  Purpose:       Start a new simulation
   In:            Sender - Standard eventhandler parameter for Delphi
   Out:
   Return:
}
var
  parameter : integer;
  InitPaused : Boolean;
begin
 {Remember whether the model was paused when play was pressed}
  InitPaused := PsMarket.Paused;
  if not PsMarket.Running then begin
 {Play one step (up to zero) to initialize input parameters}
    PsMarket.PlayStep(1);
 {Initialize input variables}
    PsMarket.Values[Parameter_Salespeople_Adjustment_Time] := AdjustmentTime;
    PsMarket.Values[Parameter_Salespeople_Salary] := StrToFloat(SalaryLabel.Caption);
    PsMarket.Values[Parameter_Revenue_To_Sales] := StrToFloat(Revenue_To_SalesLabel.Caption);
 {Play step to new autopause if autopause was checked}
    if AutoPauseCheckBox.Checked then
      PsMarket.PlayStep(StrToInt(AutoPauseEdit.Text))
    else
      PsMarket.Pause(InitPaused);
  end;
end;

procedure TDemoForm.PauseSpeedButtonClick(Sender: TObject);
{  Purpose:       Turn pause on/off.  Run to next autopause if autopause is
                  chosen
   In:            Sender - Standard eventhandler parameter for Delphi
   Out:
   Return:
}
begin
   if AutoPauseCheckBox.Checked then
     PsMarket.PlayStep(StrToInt(AutoPauseEdit.Text))
   else
     PsMarket.Pause(not PsMarket.Paused);
end;

procedure TDemoForm.StopSpeedButtonClick(Sender: TObject);
{  Purpose:       End Simulation
   In:            Sender - Standard eventhandler parameter for Delphi
   Out:
   Return:
}
begin
 {Turn pause off before stopping if the simulation is paused}
  if PsMarket.Paused then
    PsMarket.Pause(False);
  PsMarket.Play(False);
end;

procedure TDemoForm.StepSpeedButtonClick(Sender: TObject);
{  Purpose:       Run one simulation step
   In:            Sender - Standard eventhandler parameter for Delphi
   Out:
   Return:
}
begin
   PsMarket.PlayStep(1);
end;

procedure TDemoForm.PsMarketUpdateValues(Sender: TObject; Final: Boolean);
{  Purpose:       Update Powersim input and output variables and the form's
                  visual objects, such as editboxes, labels, gauges and charts.
   In:            Sender - Standard eventhandler parameter for Delphi
                  Final - Boolean parameter that tells whether the last update
                  was the final before a new time step.
   Out:
   Return:
}
var
   Parameter,PresentTime,GraphPoints : Integer;
begin
  { Update text controls of form }

   DelayLabel.Caption :=
      FormatFloat('0.00',PsMarket.Values[Parameter_Delivery_Delay_Recognized]);
   BacklogLabel.Caption :=
      FormatFloat('0.00',PsMarket.Values[Parameter_Backlog]);
   SalespeopleLabel.Caption :=
      FormatFloat('0.0',PsMarket.Values[Parameter_Salespeople]);
   BudgetLabel.Caption := FormatFloat('0.00',PsMarket.Values[Parameter_Budget]);
   OrdersLabel.Caption :=
      FormatFloat('0.00',PsMarket.Values[Parameter_Orders_Booked]);
   SalesEffLabel.Caption :=
      FormatFloat('0',PsMarket.Values[Parameter_Sales_Effectiveness]);

  { Update gauge controls of form }

   SalesEffectivenessGauge.Progress :=
      Round(PsMarket.Values[Parameter_Sales_Effectiveness]);
   TimeGauge.Progress := Round(PsMarket.Time);
   TimeLabel.Caption := FormatFloat('0.00',PsMarket.Time) + ' Weeks';

  { Update scaling of graph controls of form }

   if StrToFloat(SalespeopleLabel.Caption) > SalespeopleMaxValue then
     SalesPeopleMaxValue := StrToFloat(SalespeopleLabel.Caption);
   if StrToFloat(BacklogLabel.Caption) > BacklogMaxValue then
     BacklogMaxValue := StrToFloat(BacklogLabel.Caption);

 {Update charts and Powersim's input variables on final}
   if Final then begin
     StepsExecuted := StepsExecuted + 1;
     PresentTime := Round(PsMarket.Time);

 {Update Powersim's input variables}
    PsMarket.Values[Parameter_Salespeople_Adjustment_Time] :=
       AdjustmentTime;
    PsMarket.Values[Parameter_Revenue_To_Sales] :=
       StrToFloat(Revenue_To_SalesLabel.Caption);
    PsMarket.Values[Parameter_SalesPeople_Salary] :=
       StrToFloat(SalaryLabel.Caption);

  { Update time series (values) of graph controls of form }

    if UpdateOnEveryStep or (Frac(PsMarket.Time) = 0) then begin
      if UpDateOnEveryStep then
        GraphPoints := StepsExecuted
      else
        GraphPoints := Round(PresentTime - PsMarket.StartTime);
      with BacklogChartFX do begin
        OpenData[1] := MakeLong(1,GraphPoints);
        ThisPoint := GraphPoints;
        ThisValue := StrToFloat(BackLogLabel.Caption);
        CloseData[1] := 0;
      end;
      with SalespeopleChartFX do begin
        OpenData[1] := MakeLong(1,GraphPoints);
        ThisPoint := GraphPoints;
        ThisValue := StrToFloat(SalesPeopleLabel.Caption);
        CloseData[1] := 0;
      end;
    end;
  end;
end;

procedure TDemoForm.SalaryScrollBarScroll(Sender: TObject;
  ScrollCode: TScrollCode; var ScrollPos: Integer);
{  Purpose:       Update SalaryLabel As SalaryLabelScrollBar is scrolled
   In:            Sender - Standard eventhandler parameter for Delphi
                  ScrollCode - Delphi control parameter
                  ScrollPos - Current ScrollBar position
   Out:           ScrollPos - Current ScrollBar position after scrolling
   Return:
}
begin
   SalaryLabel.Caption := IntToStr(SalaryScrollBar.Position);
end;

procedure TDemoForm.Revenue_To_SalesScrollBarScroll(Sender: TObject;
  ScrollCode: TScrollCode; var ScrollPos: Integer);
{  Purpose:       Update Revenue_To_SalesLabel as Revenue_To_SalesScrollBar is
                  scrolled
   In:            Sender - Standard eventhandler parameter for Delphi
                  ScrollCode - Delphi control parameter
                  ScrollPos - Current ScrollBar position
   Out:           ScrollPos - Current ScrollBar position after scrolling
   Return:
}
begin
   Revenue_To_SalesLabel.Caption :=
     FloatToStr(Revenue_To_SalesScrollBar.Position/100);
end;

procedure TDemoForm.AdjustmentLongRadioButtonEnter(Sender: TObject);
{  Purpose:       Set adjustmenttime to 60 when longradiobutton is chosen
   In:            Sender - Standard eventhandler parameter for Delphi
   Out:
   Return:
}
begin
   if not AdjustmentLongRadioButton.Checked then begin
     AdjustmentLongRadioButton.Checked := True;
     AdjustmentTime := 60;
     AdjustmentTimeLabel.Caption := '60';
   end;
end;

procedure TDemoForm.AdjustmentMediumRadioButtonEnter(Sender: TObject);
{  Purpose:       Set adjustmenttime to 60 when longradiobutton is chosen
   In:            Sender - Standard eventhandler parameter for Delphi
   Out:
   Return:
}
begin
   if not AdjustmentMediumRadioButton.Checked then begin
     AdjustmentMediumRadioButton.Checked := True;
     AdjustmentTime := 20;
     AdjustmentTimeLabel.Caption := '20';
   end;
end;

procedure TDemoForm.AdjustmentShortRadioButtonEnter(Sender: TObject);
{  Purpose:       Set adjustmenttime to 60 when longradiobutton is chosen
   In:            Sender - Standard eventhandler parameter for Delphi
   Out:
   Return:
}
begin
   if not AdjustmentShortRadioButton.Checked then begin
     AdjustmentShortRadioButton.Checked := True;
     AdjustmentTime := 5;
     AdjustmentTimeLabel.Caption := '5';
   end;
end;

procedure TDemoForm.PsMarketEndSim(Sender: TObject);
{  Purpose:       Eventhandler for Powersim end simulation
   In:            Sender - Standard eventhandler parameter for Delphi
   Out:
   Return:
}
var
  i : integer;
begin
  {Scale charts}
   BacklogChartFX.Adm[1] := BacklogMaxValue;
   SalespeopleChartFX.Adm[1] := SalespeopleMaxValue;
end;

function TDemoForm.PsMarketPreBeginSim(Sender: TObject): Boolean;
{  Purpose:       Eventhandler for powersim model prebegin simulation
   In:            Sender - Standard eventhandler parameter for Delphi
   Out:
   Return:        Boolean variable, set to TRUE if simulation should stop,
                  FALSE if simulation should proceed.
}
var
  steps : double;
  i : integer;
begin
  {UPDATE PARAMETERLIST}
   if PsMarket.SearchPar('Revenue_To_Sales') = -1 then begin
     PsMarket.AppendPar('Revenue_To_Sales');
     PsMarket.AppendPar('SalesPeople_Salary');
     PsMarket.AppendPar('Backlog');
     PsMarket.AppendPar('SalesPeople');
     PsMarket.AppendPar('Orders_Booked');
     PsMarket.AppendPar('Budget');
     PsMarket.AppendPar('Sales_Effectiveness');
     PsMarket.AppendPar('SalesPeople_Adjustment_Time');
     PsMarket.AppendPar('Delivery_Delay_Recognized');

     Parameter_Revenue_To_Sales := PsMarket.SearchPar('Revenue_To_Sales');
     Parameter_Salespeople_Salary := PsMarket.SearchPar('Salespeople_Salary');
     Parameter_Backlog := PsMarket.SearchPar('Backlog');
     Parameter_Salespeople := PsMarket.SearchPar('Salespeople');
     Parameter_Orders_Booked := PsMarket.SearchPar('Orders_Booked');
     Parameter_Budget := PsMarket.SearchPar('Budget');
     Parameter_Sales_Effectiveness := PsMarket.SearchPar('Sales_Effectiveness');
     Parameter_Salespeople_Adjustment_Time :=
          PsMarket.SearchPar('Salespeople_Adjustment_Time');
     Parameter_Delivery_Delay_Recognized :=
          Psmarket.SearchPar('Delivery_Delay_Recognized');

     for i := 0 to PsMarket.ParCount-1 do
       ParametersListBox.Items.Add(PsMarket.ParNames[i]);
     ParCountEdit.Text := IntToStr(PsMarket.ParCount);
  end;
  {Update TimeGauge to new starttime and stoptime}
   TimeGauge.MinValue := Round(PsMarket.StartTime);
   TimeGauge.MaxValue := Round(PsMarket.StopTime);
   TimeGauge.Progress := Round(PsMarket.StartTime);
  {No steps executed so far}
   StepsExecuted := 0;
   Result := False;
end;

procedure TDemoForm.AutoPauseCheckBoxEnter(Sender: TObject);
{  Purpose:       Toggling the autopause property by entering the
                  AutoPauseCheckbox.
   In:            Sender - Standard eventhandler parameter for Delphi
   Out:
   Return:
}
VAR KeyPressed : Boolean;

begin
   KeyPressed := (AutoPauseCheckBox.State = cbChecked);
   Every.Enabled := KeyPressed;
   AutoPauseEdit.Enabled := KeyPressed;
   AutoPauseSpinButton.Enabled := KeyPressed;
   Step.Enabled := KeyPressed;
   AutopauseOn := KeyPressed;
end;

procedure TDemoForm.AutoPauseSpinButtonDownClick(Sender: TObject);
{  Purpose:       Decrease AutoPause step by one (PS Step must be greater than
                  or equal to one).
   In:            Sender - Standard eventhandler parameter for Delphi
   Out:
   Return:
}
Var
 CurrentValue : Integer;

begin
   CurrentValue := StrToInt(AutoPauseEdit.Text);
   if (CurrentValue-1) > 0 then
     CurrentValue := CurrentValue-1;
   AutoPauseEdit.Text := IntToStr(CurrentValue);
   AutoPauseStep := CurrentValue;
end;

procedure TDemoForm.AutoPauseSpinButtonUpClick(Sender: TObject);
{  Purpose:       Increase AutoPause step by one.
   In:            Sender - Standard eventhandler parameter for Delphi
   Out:
   Return:
}
Var
  CurrentValue : Integer;
begin
   CurrentValue := StrToInt(AutoPauseEdit.Text) + 1;
   AutoPauseEdit.Text := IntToStr(CurrentValue);
   AutoPauseStep := CurrentValue;
end;

procedure TDemoForm.LargeSalesPeopleImageClick(Sender: TObject);
{  Purpose:       Enter the reportscreen when clicking on the salespeopleimage.
   In:            Sender - Standard eventhandler parameter for Delphi
   Out:
   Return:
}
Var
  i : integer;
begin
  {Pause simulation when entering reportpage}
   if not PsMarket.Paused then
     PsMarket.Pause(True);
   MainTabbedNotebook.ActivePage := 'Report';
end;

procedure TDemoForm.BacklogBiPictClick(Sender: TObject);
{  Purpose:       Enter reportpage when clicking on backlogpicture.
   In:            Sender - Standard eventhandler parameter for Delphi
   Out:
   Return:
}
begin
  {Pause simulation when entering reportpage}
   if not PsMarket.Paused then
     PsMarket.Pause(True);
   MainTabbedNotebook.ActivePage := 'Report';
end;

procedure TDemoForm.StepEditExit(Sender: TObject);
{  Purpose:       Check for valid values when editing the time step from the
                  keyboard.
   In:            Sender - Standard eventhandler parameter for Delphi
   Out:
   Return:
}
Var
  CurrentValue : Double;
begin
   CurrentValue := StrToFloat(StepEdit.Text);
   if CurrentValue <= 0 then
   begin
     application.MessageBox('TimeStep must be > 0','Error',mb_OK);
     StepEdit.Setfocus;
   end;
end;

function TDemoForm.PsMarketBeginSim(Sender: TObject): Boolean;
{  Purpose:       Eventhandler for powersim model's begin simulation.
   In:            Sender - Standard eventhandler parameter for Delphi
   Out:
   Return:        Boolean variable, set to TRUE if simulation should stop,
                  FALSE if simulation should proceed.
}
begin
  {Reset ranges of reports}
   BacklogMaxValue := 0;
   SalesPeopleMaxValue := 0;
   Result := False;
end;

procedure TDemoForm.PsMarketCloseModel(Sender: TObject);
{  Purpose:       Empty the listbox of the model parameters when the model is
                  closed.  Empty the parametercount editbox when the model is
                  closed.
   In:            Sender - Standard eventhandler parameter for Delphi
   Out:
   Return:
}
begin
   ParametersListBox.Items.Clear;
   ParCountEdit.Text := '0';
end;

procedure TDemoForm.PsMarketOpenModel(Sender: TObject);
{  Purpose:       Initialize chart scale when the model is opened.
   In:            Sender - Standard eventhandler parameter for Delphi
   Out:
   Return:
}
begin
   ModelTitleLabel.Caption := 'Powersim Model : ' + PsMarket.ModelName;
   SalespeopleChartFX.Adm[1] := 100;
   BacklogChartFX.Adm[1] := 200000;
end;

procedure TDemoForm.CloseBitBtnClick(Sender: TObject);
{  Purpose:       Terminate the application.
   In:            Sender - Standard eventhandler parameter for Delphi
   Out:
   Return:
}
begin
  {Stop simulationg before terminating the application}
   if PsMarket.Running then
     PsMarket.Play(False);
   Application.Terminate;
end;

procedure TDemoForm.SalaryScrollBarChange(Sender: TObject);
{  Purpose:       Update salarylabel as salaryscrollbar is changed.
   In:            Sender - Standard eventhandler parameter for Delphi
   Out:
   Return:
}
begin
   SalaryLabel.Caption := IntToStr(SalaryScrollBar.Position);
end;

procedure TDemoForm.Revenue_To_SalesScrollBarChange(Sender: TObject);
{  Purpose:       Update Revenue_To_SalesLabel as Revenue_To_Sales_scrollbar
                  is changed.
   In:            Sender - Standard eventhandler parameter for Delphi
   Out:
   Return:
}
begin
   Revenue_To_SalesLabel.Caption :=
     FloatToStr(Revenue_To_SalesScrollBar.Position/100);
end;

end.

