#ifndef __PSAPI_H__
#define __PSAPI_H__

/*--------------------------------------------------------------------*/
/* The 'extern_c' macro is used to declare a name that is going       */
/* to be exported to a module not implemented in C++.                 */
/*--------------------------------------------------------------------*/


#ifdef __cplusplus
 #define extern_c extern "C"
#else
 #define extern_c extern
#endif


#ifdef WIN32
 #ifdef _PSAPIS_
  #define PSAPI_(type) extern_c type __stdcall
 #else
  #ifdef _PSAPI32_
   #define PSAPI_(type) extern_c __declspec(dllexport) type __stdcall
  #else
   #define PSAPI_(type) extern_c __declspec(dllimport) type __stdcall
  #endif
 #endif
#else
 #ifdef _PSAPIS_
  #define PSAPI_(type) extern_c type FAR PASCAL
 #else
  #define PSAPI_(type) extern_c type FAR PASCAL _export
 #endif
#endif   

/*- Callback from the simulation -------------------------------------*/
/* The following constants define the various events that will cause  */
/* a notification message to a window supplied by the client          */
/* application when opening the model.                                */
/* The notification is sent through the message WM_COPYDATA (making   */
/* 32-16 bit data transactions possible).                             */
/* During the handling of this callback, the application should not   */
/* give control back to the operating system by calling 'Yield',      */
/* 'PeekMessage', 'GetMessage' or by using DDEML for synchronous DDE. */
/* If you want to use these functions during the notification         */
/* callback, you can call 'PostMessage' and then handle the message   */
/* when Powersim is idle again.                                       */
/*--------------------------------------------------------------------*/

   #ifndef WM_COPYDATA
   
// Define the WM_COPYDATA message and accompanying structure
   #define WM_COPYDATA 0x004A
   
// The parameters in this message are the following
// wparam = (WPARAM) (HWND) hwnd ;              // Handle of sending window
// lparam - (LPARAM) (LPCOPYDATASTRUCT) lpcds ; // Pointer to structure with data
   
   typedef struct tagCOPYDATASTRUCT { // cds
      DWORD  dwData ;   // 32 bits of data to be passed to the receiving application
      DWORD  cbData ;   // Specifies the size, in bytes, of the data pointed to by the lpData member.
      LPVOID lpData ;   // Points to the data to be passed to the receiving application
   } COPYDATASTRUCT, FAR* LPCOPYDATASTRUCT ;
   #else
   typedef COPYDATASTRUCT FAR* LPCOPYDATASTRUCT ;
   #endif

// 'dwData' in the COPYDATASTRUCT is set to one of the following notifications

   #define PSAPIN_UPDATEVALUES (1)
   #define PSAPIN_DELETEPAR    (2)
   #define PSAPIN_BEGINSIM     (3)
   #define PSAPIN_ENDSIM       (4)
   #define PSAPIN_CLOSEMODEL   (5)
   #define PSAPIN_PREBEGINSIM  (6)
   #define PSAPIN_CALLBACK     (7)
   #define PSAPIN_PAUSEON      (8)
   #define PSAPIN_PAUSEOFF     (9)
   #define PSAPIN_WANTPAUSE    (10)


   #pragma pack(4)

   typedef struct tagUPDATEVALUESSTRUCT { // uvs
      DWORD    bIsFinal ;      // Is the values final values of the timestep.
      double   dbTime ;        // Current Time
      double   dbValueTab[1] ; // This is a dynamic table containing values of all parameters.
   } UPDATEVALUESSTRUCT ;


// Note that 

// lpData in the COPYDATASTRUCT points to the following structure containing additional notification info.
   typedef struct tagNOTIFICATIONDATASTRUCT { // nds
      LPVOID   lpUserData ;            // Parameter to 'OpenModel'
      union {
         UPDATEVALUESSTRUCT uvs ;      // Used with the PSAPIN_UPDATEVALUES notification
         long               lIndex ;  // Used with the PSAPIN_DELETEPAR notification
      } ;
   } NOTIFICATIONDATASTRUCT, FAR* LPNOTIFICATIONDATASTRUCT ;

   #pragma pack()

// The return value from the notification message is ignored except in the case of
// PSAPIN_BEGINSIM and PSAPIN_PREBEGINSIM where returning TRUE will stop the simulation.

/*- Notification Callback --------------------------------------------*/
/* When the client application receives the 'PSAPIN_CALLBACK'         */
/* notification, it should call the 'PSAPI_NotificationCallback'      */
/* function with 'lpData' and 'cbData' set to the corresponding       */
/* values in the 'COPYDATASTRUCT' sent with the message.              */
/*--------------------------------------------------------------------*/

   PSAPI_(LRESULT) PSAPI_NotificationCallback(LPVOID lpData, DWORD cbData) ;

/*- Opening and closing the model. -----------------------------------*/
/* Use 'PSAPI_OpenModel' to open a connection to a Powersim model.    */
/* The returned handle is used with the other functions, and is valid */
/* until 'PSAPI_CloseModel' is called, or until the application has   */
/* been notified that the model is closed through the notification    */
/* message 'PSAPIN_CLOSEMODEL'.                                       */
/* Note that 'PSAPI_CloseModel ' must be called if the window         */
/* registered with 'PSAPI_OpenModel' is deleted or of some other      */
/* reason no longer handles the notification messages.                */
/* Using an invalid handle can cause unpredictable results, Powersim  */
/* will most likely cause a GPF.                                      */
/* Call 'PSAPI_EnableWantPauseNotifications' to enable your           */
/* application to pause the simulation when certain events occurs in  */
/* the model being simulated (Is disabled by default.)                */
/*--------------------------------------------------------------------*/

   #define PSAPI_PATH_SIZE 256

// Handle to model returned by 'PSAPI_OpenModel' and used in subsequent calls to identify the model.
   typedef long HCONNECTION ;

// Open named Powersim document and return handle to model (0 if failure)
   PSAPI_(HCONNECTION) PSAPI_OpenConnection(LPCSTR lpszDocName, LPSTR lpszBrowsedPath,
      HWND hwndApp, BOOL bCopyValuesOnUpdate, void FAR* lpUserData) ;
   
// Close model
   PSAPI_(BOOL) PSAPI_CloseConnection(HCONNECTION hConnection) ;

   PSAPI_(BOOL) PSAPI_EnableWantPauseNotifications(HCONNECTION hConnection, BOOL bEnable) ;


/*- Handling connection parameters -----------------------------------*/
/* The Powersim model is controlled and monitored through parameters  */
/* linked to variables in the model.                                  */
/* These functions makes it possible to add and remove parameters to  */
/* the connection.                                                    */
/* Note that if a parameter with an index 'i' is removed, all the     */
/* parameters with index 'j', where 'j' > 'i', are shifted forward,   */
/* and must be accessed with the index 'j-1'.                         */
/*--------------------------------------------------------------------*/

// Append parameter, return zero based index, or -1 if function failed.
   PSAPI_(int) PSAPI_AppendPar(HCONNECTION hConnection, const char FAR *lpszParName) ;
   
// Remove parameter at given position
   PSAPI_(BOOL) PSAPI_RemoveParAt(HCONNECTION hConnection, int iPar) ;
   
// Remove all parameters
   PSAPI_(BOOL) PSAPI_RemoveAllPar(HCONNECTION hConnection) ;


/*- Parameter List Observervers --------------------------------------*/
/* The following functions can be used to observe the parameters in   */
/* the connection to the Powersim model.                              */
/*--------------------------------------------------------------------*/

// Get number of parameters
   PSAPI_(int) PSAPI_GetParCount(HCONNECTION hConnection) ;

// Get name of parameter (store in 'lpszName' buffer, also returned by function)
   PSAPI_(LPSTR) PSAPI_GetParName(HCONNECTION hConnection, int iPar, 
      char FAR * lpszName, UINT uiSizeOfName) ;
   
// Search for named parameter. Return zero based index (-1 if not found)
   PSAPI_(int) PSAPI_SearchPar(HCONNECTION hConnection, const char FAR *lpszParName) ;

// Get unit of parameter (store in 'lpszUnit' buffer, also returned by function)
   PSAPI_(LPSTR) PSAPI_GetParUnit(HCONNECTION hConnection, int iPar, 
      char FAR * lpszUnit, UINT uiSizeOfUnit) ;

// Get scale of parameter 
   PSAPI_(BOOL) PSAPI_GetParScale(HCONNECTION hConnection, int iPar, 
      double FAR *pdbMin, double FAR *pdbMax) ;


/*- Getting and Setting Values in Model ------------------------------*/
/* The reason that there are two functions to get values from the     */
/* model, is that communication between 32 and 16 bits applications   */
/* takes considerable time on Windows NT due to several context       */
/* switches.                                                          */
/* By using only four byte precision we reduce the number of needed   */
/* context switches.                                                  */
/*--------------------------------------------------------------------*/
   
// Get value of parameter at given index
   PSAPI_(BOOL) PSAPI_GetParValue(HCONNECTION hConnection, int iPar, double FAR *pdbValue) ;
   PSAPI_(float) PSAPI_GetShortParValue(HCONNECTION hConnection, int iPar) ;
   
// Set value of parameter at given index
   PSAPI_(BOOL) PSAPI_SetParValue(HCONNECTION hConnection, int iPar, double dbValue) ;


/*- Control Values ---------------------------------------------------*/
/* PSAPI has two functions for setting and retrieving control values  */
/* of the Powersim model.                                             */
/* Both functions return TRUE if the operation succeeded and FALSE if */
/* an error occurred.                                                 */
/* The constants prefixed with 'PSAPI_CTL' identify the control       */
/* parameter to be used.                                              */
/*--------------------------------------------------------------------*/

   PSAPI_(BOOL) PSAPI_SetControlValue(HCONNECTION hConnection, int nCtlID, double dbValue);
   PSAPI_(BOOL) PSAPI_GetControlValue(HCONNECTION hConnection, int nCtlID, double FAR *pdbValue);

// The folllowing control parameter identifiers can be used with the functions above:
   #define PSAPI_CTL_PLAY        0
   #define PSAPI_CTL_PAUSE       1
   #define PSAPI_CTL_TIME        2
   #define PSAPI_CTL_TIMESTEP    3
   #define PSAPI_CTL_STARTTIME   4
   #define PSAPI_CTL_STOPTIME    5
   #define PSAPI_CTL_TIMEDELAY   6
   #define PSAPI_CTL_PAUSEATTIME 7
   #define PSAPI_CTL_PLAYSTEP    8

/*- Changing Ranges in a Powersim Model ------------------------------*/
/* PSAPI contains functions to change the definition of numeric       */
/* ranges in a Powersim model.                                        */
/* To be able to use these functions, the Powersim model must not     */
/* allow edit access (uncheck Edit access in the Access Dialog found  */
/* in the Properties Dialog).                                         */
/* All functions return whether they succeeded or not.                */
/* First call 'PSAPI_BeginChangeRanges', and then an arbitarty number */
/* of 'PSAPI_SetRange' before finishing the session by calling        */
/* 'PSAPI_EndChangeRanges'.                                           */
/* Note that the model will be inconsistent if you don't finish the   */
/* session after one or more calls to 'PSAPI_SetRange'.               */
/* These functions cannot be used while simulating (can be used when  */
/* handling the 'PSAPIN_PREBEGINSIM' notification).                   */
/*--------------------------------------------------------------------*/

   PSAPI_(BOOL) PSAPI_BeginChangeRanges(HCONNECTION hConnection) ;
   PSAPI_(BOOL) PSAPI_EndChangeRanges(HCONNECTION hConnection) ;
   PSAPI_(BOOL) PSAPI_SetRange(HCONNECTION hConnection, LPCSTR lpszRangeName,
      short nFirst, short nLast) ;

/*- Retrieving information about variables in a powersim model -------*/
/* PSAPI contains functions to enable collection of some information  */
/* about the models and variables of a simulation.                    */ 
/* 'PSAPI_GetModelCount' returns the total number of models used by   */
/* a main model (including itself), and 'PSAPI_GetModelName' gets the */
/* names of these models.                                             */
/* Similarly will 'PSAPI_GetVariableCount' retrieve the total number  */
/* of variables in one of these models, and 'PSAPI_GetVariableName'   */
/* get their names.                                                   */
/* To enumerate the elements of these variables use                   */
/* 'PSAPI_GetElementCount' and 'PSAPI_GetElementName'.                */
/* Notice that 'PSAPI_GetElementCount' returns 0 for scalars.         */
/* 'PSAPI_VariableHasInputs' checks whether the variable is           */
/* dependent of other variables, and 'PSAPI_GetVariableType' returns  */
/* the variable type.                                                 */
/*--------------------------------------------------------------------*/

   // Variable types
   #define PSAPI_VT_LEVEL 1
   #define PSAPI_VT_CONST 2
   #define PSAPI_VT_AUX   3
   
   PSAPI_(int) PSAPI_GetModelCount(HCONNECTION hConnection) ;
   PSAPI_(LPSTR) PSAPI_GetModelName(HCONNECTION hConnection, int iModel, 
      LPSTR lpszName, UINT cbNameBuf) ;

   PSAPI_(int) PSAPI_GetVariableCount(HCONNECTION hConnection, int iModel) ;
   PSAPI_(LPSTR) PSAPI_GetVariableName(HCONNECTION hConnection, int iModel,
      int iVariable, LPSTR lpszName, UINT cbNameBuf) ;

   PSAPI_(int) PSAPI_VariableHasInputs(HCONNECTION hConnection, int iModel, int iVariable) ;
   PSAPI_(int) PSAPI_GetVariableType(HCONNECTION hConnection, int iModel, int iVariable) ;

   PSAPI_(LPSTR) PSAPI_GetElementName(HCONNECTION hConnection, int iModel,
      int iVariable, int iElement, LPSTR lpszName, UINT cbNameBuf) ;
   PSAPI_(int) PSAPI_GetElementCount(HCONNECTION hConnection, int iModel, int iVariable) ;

#endif // __PSAPI_H__
      