{*******************************************************}
{                                                       }
{       Powersim API for Delphi                         }
{                                                       }
{       Copyright (c) 1996 Powersim AS                  }
{                                                       }
{				This is the Delphi equivalent of the C-style    }
{       header file PSAPI.H. See this header file for   }
{       more detailed information.                      }
{                                                       }
{*******************************************************}

unit PsApi;

interface

uses WinTypes;

const

{ The WM_COPYDATA message is used for notification to the connected application }

	WM_COPYDATA = $4A;

type

{ The parameters of the WM_COPYDATA message are: }
{ wParam - SendingWnd: HWnd }
{ lParam - CopyDataPtr: PCopyDataStruct }

	PCopyDataStruct = ^TCopyDataStruct;
	TCopyDataStruct = record
		Data: Longint;  { 32 bits of data to be passed to the receiving application }
    DataSize: Longint;  { Specifies the size, in bytes, of the data pointed to by DataPtr }
    DataPtr: Pointer;  { Points to the data to be passed to the receiving application }
  end;

{ The WM_COPYDATA message struct }

  TWMCopyData = record
    Msg: Cardinal;
    SendingWnd: HWnd;
    CopyDataPtr: PCopyDataStruct;
    Result: Longint;
  end;

const

{ The Data field in the TCopyDataStruct is set to one of the following notifications }

  PSAPI_NFC_UPDATEVALUES = 1;
  PSAPI_NFC_DELETEPAR = 2;
  PSAPI_NFC_BEGINSIM = 3;
  PSAPI_NFC_ENDSIM = 4;
  PSAPI_NFC_CLOSEMODEL = 5;
  PSAPI_NFC_PREBEGINSIM = 6;
  PSAPI_NFC_CALLBACK = 7;
  PSAPI_NFC_PAUSEON = 8;
  PSAPI_NFC_PAUSEOFF = 9;
  PSAPI_NFC_WANTPAUSE = 10;

type

{ The record below is used in connection with UpdateValues and WantPause notifications }

  TUpdateValueStruct = record
  	IsFinal: Longint;  { Is it the final values of the timestep? }
    Time: Double; 		 { Current time }
    ValueTable: array [0..0] of Double;  { This is a dynamic table containing values of all parameters }
  end;

{ DataPtr in TCopyDataStruct points to the following structure containing additional notification info }

	PNotificationDataStruct = ^TNotificationDataStruct;
	TNotificationDataStruct = record
		UserDataPtr: Pointer;  { The user data pointer from PSAPI_OpenConnection }
	  case Integer of
  	  0: (UpdateValueStruct: TUpdateValueStruct);  { Used with the PSAPI_NFC_UPDATEVALUES notification }
    	1: (Index: Longint);  { Used with the PSAPI_NFC_DELETEPAR notification }
	end;

{ The return value from the notification message is ignored except in the case of }
{ PSAPI_NFC_BEGINSIM and PSAPI_NFC_PREBEGINSIM where returning FALSE will stop the simulation }

const

{ Control value IDs }

  PSAPI_CTL_PLAY = 0;
  PSAPI_CTL_PAUSE = 1;
  PSAPI_CTL_TIME = 2;
  PSAPI_CTL_TIMESTEP = 3;
  PSAPI_CTL_STARTTIME = 4;
  PSAPI_CTL_STOPTIME = 5;
  PSAPI_CTL_TIMEDELAY = 6;
  PSAPI_CTL_PAUSEATTIME = 7;
  PSAPI_CTL_PLAYSTEP = 8;

{ Variable types }

	PSAPI_VT_LEVEL = 1;
  PSAPI_VT_CONST = 2;
  PSAPI_VT_AUX = 3;

type
  HConnection = Longint;
	PDouble = ^Double;

function PSAPI_OpenConnection(ModelName: PChar; BrowsedPath: PChar;
	NotifyWnd: HWnd; CopyValuesOnUpdate: Boolean; DataPtr: Pointer): HConnection;
function PSAPI_CloseConnection(Connection: HConnection): Boolean;
function PSAPI_EnableWantPauseNotifications(Connection: HConnection; Enable: Boolean): Boolean;

function PSAPI_AppendPar(Connection: HConnection; ParName: PChar): Integer;
function PSAPI_RemoveParAt(Connection: HConnection; Index: Integer): Boolean;
function PSAPI_RemoveAllPar(Connection: HConnection): Boolean;

function PSAPI_GetParCount(Connection: HConnection): Integer;
function PSAPI_GetParName(Connection: HConnection; Index: Integer;
	Name: PChar; SizeOfName: Word): PChar ;
function PSAPI_SearchPar(Connection: HConnection; ParName: PChar): Integer;
function PSAPI_GetParUnit(Connection: HConnection; Index: Integer;
	UnitName: PChar; SizeOfUnit: Word): PChar ;
function PSAPI_GetParScale(Connection: HConnection; Index: Integer;
	MinPtr: PDouble; MaxPtr: PDouble): Boolean ;

function PSAPI_GetParValue(Connection: HConnection; Index: Integer; ValuePtr: PDouble): Boolean;
function PSAPI_SetParValue(Connection: HConnection; Index: Integer; Value: Double): Boolean;

function PSAPI_SetControlValue(Connection: HConnection; CtlID: Integer; Value: Double): Boolean;
function PSAPI_GetControlValue(Connection: HConnection; CtlID: Integer; ValuePtr: PDouble): Boolean;

function PSAPI_BeginChangeRanges(Connection: HConnection): Boolean;
function PSAPI_EndChangeRanges(Connection: HConnection): Boolean;
function PSAPI_SetRange(Connection: HConnection; RangeName: PChar; First: Integer; Last: Integer): Boolean;

function PSAPI_GetModelCount(Connection: HConnection): Integer ;
function PSAPI_GetModelName(Connection: HConnection; ModelIdx: Integer;
	ModelName: PChar; SizeOfName: Word): PChar ;

function PSAPI_GetVariableCount(Connection: HConnection; ModelIdx: Integer): Integer;
function PSAPI_GetVariableName(Connection: HConnection; ModelIdx: Integer;
	VariableIdx: Integer; VariableName: PChar; SizeOfName: Word): PChar;

function PSAPI_VariableHasInputs(Connection: HConnection; ModelIdx: Integer;
	VariableIdx: Integer): Integer;
function PSAPI_GetVariableType(Connection: HConnection; ModelIdx: Integer;
	VariableIdx: Integer): Integer;

function PSAPI_GetElementName(Connection: HConnection; ModelIdx: Integer;
	VariableIdx: Integer; ElementIdx: Integer; ElementName: PChar; SizeOfName: Word): PChar;
function PSAPI_GetElementCount(Connection: HConnection; ModelIdx: Integer;
	VariableIdx: Integer): Integer;

function PSAPI_NotificationCallback(DataPtr: Pointer; DataSize: Longint): Longint;

implementation

function PSAPI_OpenConnection;		external 'PSAPI16' index 10;
function PSAPI_CloseConnection;		external 'PSAPI16' index 11;
function PSAPI_GetParCount;				external 'PSAPI16' index 12;
function PSAPI_GetParName;				external 'PSAPI16' index 13;
function PSAPI_SearchPar;					external 'PSAPI16' index 14;
function PSAPI_AppendPar;					external 'PSAPI16' index 15;
function PSAPI_RemoveParAt;				external 'PSAPI16' index 16;
function PSAPI_RemoveAllPar;			external 'PSAPI16' index 17;
function PSAPI_GetParValue;				external 'PSAPI16' index 18;
function PSAPI_SetParValue;				external 'PSAPI16' index 19;
function PSAPI_BeginChangeRanges;	external 'PSAPI16' index 20;
function PSAPI_EndChangeRanges;		external 'PSAPI16' index 21;
function PSAPI_SetRange;					external 'PSAPI16' index 22;
function PSAPI_SetControlValue;		external 'PSAPI16' index 23;
function PSAPI_GetControlValue;		external 'PSAPI16' index 24;
function PSAPI_GetParUnit;				external 'PSAPI16' index 26;
function PSAPI_GetParScale;				external 'PSAPI16' index 27;
function PSAPI_NotificationCallback;          external 'PSAPI16' index 29;
function PSAPI_EnableWantPauseNotifications;	external 'PSAPI16' index 30;
function PSAPI_GetModelCount;			external 'PSAPI16' index 32;
function PSAPI_GetModelName;			external 'PSAPI16' index 33;
function PSAPI_GetVariableCount;	external 'PSAPI16' index 34;
function PSAPI_GetVariableName;		external 'PSAPI16' index 35;
function PSAPI_VariableHasInputs;	external 'PSAPI16' index 36;
function PSAPI_GetVariableType;		external 'PSAPI16' index 37;
function PSAPI_GetElementName;		external 'PSAPI16' index 38;
function PSAPI_GetElementCount;		external 'PSAPI16' index 39;

end.
